@@ -2,6 +2,7 @@ import html
 import json
 import warnings
 from datetime import timedelta
+from urllib.parse import urlparse
 
 from django import forms
 from django.conf import settings
@@ -15,6 +16,7 @@ from django.contrib.auth import (
 from django.contrib.auth.models import AbstractUser
 from django.contrib.auth.password_validation import validate_password
 from django.contrib.sites.shortcuts import get_current_site
+from django.core.exceptions import FieldDoesNotExist
 from django.core.mail import EmailMessage, EmailMultiAlternatives
 from django.http import HttpResponse, HttpResponseRedirect
 from django.shortcuts import resolve_url
@@ -31,7 +33,6 @@ from allauth.account import signals
 from allauth.account.app_settings import EmailVerificationMethod
 from allauth.utils import (
     build_absolute_uri,
-    email_address_exists,
     generate_unique_username,
     get_user_model,
     import_attribute,
@@ -41,7 +42,6 @@ from . import app_settings
 
 
 class DefaultAccountAdapter(object):
-
     error_messages = {
         "username_blacklisted": _(
             "Username can not be used. Please use other username."
@@ -52,7 +52,7 @@ class DefaultAccountAdapter(object):
         "too_many_login_attempts": _(
             "Too many failed login attempts. Try again later."
         ),
-        "email_taken": _("A user is already registered with this e-mail address."),
+        "email_taken": _("A user is already registered with this email address."),
     }
 
     def __init__(self, request=None):
@@ -93,15 +93,15 @@ class DefaultAccountAdapter(object):
 
     def get_from_email(self):
         """
-        This is a hook that can be overridden to programatically
+        This is a hook that can be overridden to programmatically
         set the 'from' email address for sending emails
         """
         return settings.DEFAULT_FROM_EMAIL
 
     def render_mail(self, template_prefix, email, context, headers=None):
         """
-        Renders an e-mail to `email`.  `template_prefix` identifies the
-        e-mail that is to be sent, e.g. "account/email/email_confirmation"
+        Renders an email to `email`.  `template_prefix` identifies the
+        email that is to be sent, e.g. "account/email/email_confirmation"
         """
         to = [email] if isinstance(email, str) else email
         subject = render_to_string("{0}_subject.txt".format(template_prefix), context)
@@ -171,7 +171,7 @@ class DefaultAccountAdapter(object):
 
     def get_email_confirmation_redirect_url(self, request):
         """
-        The URL to return to after successful e-mail confirmation.
+        The URL to return to after successful email confirmation.
         """
         if request.user.is_authenticated:
             if app_settings.EMAIL_CONFIRMATION_AUTHENTICATED_REDIRECT_URL:
@@ -307,7 +307,9 @@ class DefaultAccountAdapter(object):
         return password
 
     def validate_unique_email(self, email):
-        if email_address_exists(email):
+        from .models import EmailAddress
+
+        if EmailAddress.objects.is_verified(email):
             raise forms.ValidationError(self.error_messages["email_taken"])
         return email
 
@@ -473,25 +475,51 @@ class DefaultAccountAdapter(object):
         """
         Marks the email address as confirmed on the db
         """
-        email_address.verified = True
-        email_address.set_as_primary(conditional=True)
-        email_address.save()
+        from allauth.account.models import EmailAddress
+
+        from_email_address = (
+            EmailAddress.objects.filter(user_id=email_address.user_id)
+            .exclude(pk=email_address.pk)
+            .first()
+        )
+        if not email_address.set_verified(commit=False):
+            return False
+        email_address.set_as_primary(conditional=(not app_settings.CHANGE_EMAIL))
+        email_address.save(update_fields=["verified", "primary"])
+        if app_settings.CHANGE_EMAIL:
+            for instance in EmailAddress.objects.filter(
+                user_id=email_address.user_id
+            ).exclude(pk=email_address.pk):
+                instance.remove()
+            signals.email_changed.send(
+                sender=get_user_model(),
+                request=request,
+                user=email_address.user,
+                from_email_address=from_email_address,
+                to_email_address=email_address,
+            )
+        return True
 
     def set_password(self, user, password):
         user.set_password(password)
         user.save()
 
     def get_user_search_fields(self):
-        user = get_user_model()()
-        return filter(
-            lambda a: a and hasattr(user, a),
-            [
-                app_settings.USER_MODEL_USERNAME_FIELD,
-                "first_name",
-                "last_name",
-                "email",
-            ],
-        )
+        ret = []
+        User = get_user_model()
+        candidates = [
+            app_settings.USER_MODEL_USERNAME_FIELD,
+            "first_name",
+            "last_name",
+            "email",
+        ]
+        for candidate in candidates:
+            try:
+                User._meta.get_field(candidate)
+                ret.append(candidate)
+            except FieldDoesNotExist:
+                pass
+        return ret
 
     def is_safe_url(self, url):
         try:
@@ -501,7 +529,15 @@ class DefaultAccountAdapter(object):
                 is_safe_url as url_has_allowed_host_and_scheme,
             )
 
-        return url_has_allowed_host_and_scheme(url, allowed_hosts=None)
+        # get_host already validates the given host, so no need to check it again
+        allowed_hosts = {self.request.get_host()} | set(settings.ALLOWED_HOSTS)
+
+        if "*" in allowed_hosts:
+            parsed_host = urlparse(url).netloc
+            allowed_host = {parsed_host} if parsed_host else None
+            return url_has_allowed_host_and_scheme(url, allowed_hosts=allowed_host)
+
+        return url_has_allowed_host_and_scheme(url, allowed_hosts=allowed_hosts)
 
     def get_email_confirmation_url(self, request, emailconfirmation):
         """Constructs the email confirmation (activation) url.
@@ -522,7 +558,7 @@ class DefaultAccountAdapter(object):
             send_email = ratelimit.consume(
                 request,
                 action="confirm_email",
-                key=email_address.email,
+                key=email_address.email.lower(),
                 amount=1,
                 duration=cooldown_period.total_seconds(),
             )
@@ -533,6 +569,20 @@ class DefaultAccountAdapter(object):
             ).exists()
         return send_email
 
+    def send_account_already_exists_mail(self, email):
+        signup_url = build_absolute_uri(self.request, reverse("account_signup"))
+        password_reset_url = build_absolute_uri(
+            self.request, reverse("account_reset_password")
+        )
+        context = {
+            "request": self.request,
+            "current_site": get_current_site(self.request),
+            "email": email,
+            "signup_url": signup_url,
+            "password_reset_url": password_reset_url,
+        }
+        self.send_mail("account/email/account_already_exists", email, context)
+
     def send_confirmation_mail(self, request, emailconfirmation, signup):
         current_site = get_current_site(request)
         activate_url = self.get_email_confirmation_url(request, emailconfirmation)
