@@ -1,43 +1,57 @@
-class AppSettings(object):
+from django.core.exceptions import ImproperlyConfigured
 
+
+class AppSettings(object):
     class AuthenticationMethod:
-        USERNAME = 'username'
-        EMAIL = 'email'
-        USERNAME_EMAIL = 'username_email'
+        USERNAME = "username"
+        EMAIL = "email"
+        USERNAME_EMAIL = "username_email"
 
     class EmailVerificationMethod:
         # After signing up, keep the user account inactive until the email
         # address is verified
-        MANDATORY = 'mandatory'
-        # Allow login with unverified e-mail (e-mail verification is
+        MANDATORY = "mandatory"
+        # Allow login with unverified email (email verification is
         # still sent)
-        OPTIONAL = 'optional'
-        # Don't send e-mail verification mails during signup
-        NONE = 'none'
+        OPTIONAL = "optional"
+        # Don't send email verification mails during signup
+        NONE = "none"
 
     def __init__(self, prefix):
         self.prefix = prefix
         # If login is by email, email must be required
-        assert (not self.AUTHENTICATION_METHOD ==
-                self.AuthenticationMethod.EMAIL) or self.EMAIL_REQUIRED
+        assert (
+            not self.AUTHENTICATION_METHOD == self.AuthenticationMethod.EMAIL
+        ) or self.EMAIL_REQUIRED
         # If login includes email, login must be unique
-        assert (self.AUTHENTICATION_METHOD ==
-                self.AuthenticationMethod.USERNAME) or self.UNIQUE_EMAIL
-        assert (self.EMAIL_VERIFICATION !=
-                self.EmailVerificationMethod.MANDATORY) \
-            or self.EMAIL_REQUIRED
+        assert (
+            self.AUTHENTICATION_METHOD == self.AuthenticationMethod.USERNAME
+        ) or self.UNIQUE_EMAIL
+        assert (
+            self.EMAIL_VERIFICATION != self.EmailVerificationMethod.MANDATORY
+        ) or self.EMAIL_REQUIRED
         if not self.USER_MODEL_USERNAME_FIELD:
             assert not self.USERNAME_REQUIRED
-            assert self.AUTHENTICATION_METHOD \
-                not in (self.AuthenticationMethod.USERNAME,
-                        self.AuthenticationMethod.USERNAME_EMAIL)
+            assert self.AUTHENTICATION_METHOD not in (
+                self.AuthenticationMethod.USERNAME,
+                self.AuthenticationMethod.USERNAME_EMAIL,
+            )
+        if self.MAX_EMAIL_ADDRESSES is not None:
+            assert self.MAX_EMAIL_ADDRESSES > 0
+        if self.CHANGE_EMAIL:
+            if self.MAX_EMAIL_ADDRESSES is not None and self.MAX_EMAIL_ADDRESSES != 2:
+                raise ImproperlyConfigured(
+                    "Invalid combination of ACCOUNT_CHANGE_EMAIL and ACCOUNT_MAX_EMAIL_ADDRESSES"
+                )
 
     def _setting(self, name, dflt):
-        from django.conf import settings
-        getter = getattr(settings,
-                         'ALLAUTH_SETTING_GETTER',
-                         lambda name, dflt: getattr(settings, name, dflt))
-        return getter(self.prefix + name, dflt)
+        from allauth.utils import get_setting
+
+        return get_setting(self.prefix + name, dflt)
+
+    @property
+    def PREVENT_ENUMERATION(self):
+        return self._setting("PREVENT_ENUMERATION", True)
 
     @property
     def DEFAULT_HTTP_PROTOCOL(self):
@@ -46,31 +60,35 @@ class AppSettings(object):
     @property
     def EMAIL_CONFIRMATION_EXPIRE_DAYS(self):
         """
-        Determines the expiration date of e-mail confirmation mails (#
+        Determines the expiration date of email confirmation mails (#
         of days)
         """
         from django.conf import settings
-        return self._setting("EMAIL_CONFIRMATION_EXPIRE_DAYS",
-                             getattr(settings, "EMAIL_CONFIRMATION_DAYS", 3))
+
+        return self._setting(
+            "EMAIL_CONFIRMATION_EXPIRE_DAYS",
+            getattr(settings, "EMAIL_CONFIRMATION_DAYS", 3),
+        )
 
     @property
     def EMAIL_CONFIRMATION_AUTHENTICATED_REDIRECT_URL(self):
         """
-        The URL to redirect to after a successful e-mail confirmation, in
+        The URL to redirect to after a successful email confirmation, in
         case of an authenticated user
         """
-        return self._setting("EMAIL_CONFIRMATION_AUTHENTICATED_REDIRECT_URL",
-                             None)
+        return self._setting("EMAIL_CONFIRMATION_AUTHENTICATED_REDIRECT_URL", None)
 
     @property
     def EMAIL_CONFIRMATION_ANONYMOUS_REDIRECT_URL(self):
         """
-        The URL to redirect to after a successful e-mail confirmation, in
+        The URL to redirect to after a successful email confirmation, in
         case no user is logged in
         """
         from django.conf import settings
-        return self._setting("EMAIL_CONFIRMATION_ANONYMOUS_REDIRECT_URL",
-                             settings.LOGIN_URL)
+
+        return self._setting(
+            "EMAIL_CONFIRMATION_ANONYMOUS_REDIRECT_URL", settings.LOGIN_URL
+        )
 
     @property
     def EMAIL_CONFIRMATION_COOLDOWN(self):
@@ -83,17 +101,16 @@ class AppSettings(object):
     @property
     def EMAIL_REQUIRED(self):
         """
-        The user is required to hand over an e-mail address when signing up
+        The user is required to hand over an email address when signing up
         """
         return self._setting("EMAIL_REQUIRED", False)
 
     @property
     def EMAIL_VERIFICATION(self):
         """
-        See e-mail verification method
+        See email verification method
         """
-        ret = self._setting("EMAIL_VERIFICATION",
-                            self.EmailVerificationMethod.OPTIONAL)
+        ret = self._setting("EMAIL_VERIFICATION", self.EmailVerificationMethod.OPTIONAL)
         # Deal with legacy (boolean based) setting
         if ret is True:
             ret = self.EmailVerificationMethod.MANDATORY
@@ -101,23 +118,30 @@ class AppSettings(object):
             ret = self.EmailVerificationMethod.OPTIONAL
         return ret
 
+    @property
+    def MAX_EMAIL_ADDRESSES(self):
+        return self._setting("MAX_EMAIL_ADDRESSES", None)
+
+    @property
+    def CHANGE_EMAIL(self):
+        return self._setting("CHANGE_EMAIL", False)
+
     @property
     def AUTHENTICATION_METHOD(self):
-        ret = self._setting("AUTHENTICATION_METHOD",
-                            self.AuthenticationMethod.USERNAME)
+        ret = self._setting("AUTHENTICATION_METHOD", self.AuthenticationMethod.USERNAME)
         return ret
 
     @property
     def EMAIL_MAX_LENGTH(self):
         """
-        Adjust max_length of e-mail addresses
+        Adjust max_length of email addresses
         """
         return self._setting("EMAIL_MAX_LENGTH", 254)
 
     @property
     def UNIQUE_EMAIL(self):
         """
-        Enforce uniqueness of e-mail addresses
+        Enforce uniqueness of email addresses
         """
         return self._setting("UNIQUE_EMAIL", True)
 
@@ -133,8 +157,14 @@ class AppSettings(object):
         """
         Signup password verification
         """
-        legacy = self._setting('SIGNUP_PASSWORD_VERIFICATION', True)
-        return self._setting('SIGNUP_PASSWORD_ENTER_TWICE', legacy)
+        legacy = self._setting("SIGNUP_PASSWORD_VERIFICATION", True)
+        return self._setting("SIGNUP_PASSWORD_ENTER_TWICE", legacy)
+
+    @property
+    def SIGNUP_REDIRECT_URL(self):
+        from django.conf import settings
+
+        return self._setting("SIGNUP_REDIRECT_URL", settings.LOGIN_REDIRECT_URL)
 
     @property
     def PASSWORD_MIN_LENGTH(self):
@@ -142,11 +172,31 @@ class AppSettings(object):
         Minimum password Length
         """
         from django.conf import settings
+
         ret = None
         if not settings.AUTH_PASSWORD_VALIDATORS:
             ret = self._setting("PASSWORD_MIN_LENGTH", 6)
         return ret
 
+    @property
+    def RATE_LIMITS(self):
+        dflt = {
+            # Change password view (for users already logged in)
+            "change_password": "5/m",
+            # Email management (e.g. add, remove, change primary)
+            "manage_email": "10/m",
+            # Request a password reset, global rate limit per IP
+            "reset_password": "20/m",
+            # Rate limit measured per individual email address
+            "reset_password_email": "5/m",
+            # Password reset (the view the password reset email links to).
+            "reset_password_from_key": "20/m",
+            # Signups.
+            "signup": "20/m",
+            # NOTE: Login is already protected via `ACCOUNT_LOGIN_ATTEMPTS_LIMIT`
+        }
+        return self._setting("RATE_LIMITS", dflt)
+
     @property
     def EMAIL_SUBJECT_PREFIX(self):
         """
@@ -191,23 +241,22 @@ class AppSettings(object):
 
     @property
     def ADAPTER(self):
-        return self._setting('ADAPTER',
-                             'allauth.account.adapter.DefaultAccountAdapter')
+        return self._setting("ADAPTER", "allauth.account.adapter.DefaultAccountAdapter")
 
     @property
     def CONFIRM_EMAIL_ON_GET(self):
-        return self._setting('CONFIRM_EMAIL_ON_GET', False)
+        return self._setting("CONFIRM_EMAIL_ON_GET", False)
 
     @property
     def AUTHENTICATED_LOGIN_REDIRECTS(self):
-        return self._setting('AUTHENTICATED_LOGIN_REDIRECTS', True)
+        return self._setting("AUTHENTICATED_LOGIN_REDIRECTS", True)
 
     @property
     def LOGIN_ON_EMAIL_CONFIRMATION(self):
         """
         Automatically log the user in once they confirmed their email address
         """
-        return self._setting('LOGIN_ON_EMAIL_CONFIRMATION', False)
+        return self._setting("LOGIN_ON_EMAIL_CONFIRMATION", False)
 
     @property
     def LOGIN_ON_PASSWORD_RESET(self):
@@ -215,27 +264,29 @@ class AppSettings(object):
         Automatically log the user in immediately after resetting
         their password.
         """
-        return self._setting('LOGIN_ON_PASSWORD_RESET', False)
+        return self._setting("LOGIN_ON_PASSWORD_RESET", False)
 
     @property
     def LOGOUT_REDIRECT_URL(self):
-        return self._setting('LOGOUT_REDIRECT_URL', '/')
+        from django.conf import settings
+
+        return self._setting("LOGOUT_REDIRECT_URL", settings.LOGOUT_REDIRECT_URL or "/")
 
     @property
     def LOGOUT_ON_GET(self):
-        return self._setting('LOGOUT_ON_GET', False)
+        return self._setting("LOGOUT_ON_GET", False)
 
     @property
     def LOGOUT_ON_PASSWORD_CHANGE(self):
-        return self._setting('LOGOUT_ON_PASSWORD_CHANGE', False)
+        return self._setting("LOGOUT_ON_PASSWORD_CHANGE", False)
 
     @property
     def USER_MODEL_USERNAME_FIELD(self):
-        return self._setting('USER_MODEL_USERNAME_FIELD', 'username')
+        return self._setting("USER_MODEL_USERNAME_FIELD", "username")
 
     @property
     def USER_MODEL_EMAIL_FIELD(self):
-        return self._setting('USER_MODEL_EMAIL_FIELD', 'email')
+        return self._setting("USER_MODEL_EMAIL_FIELD", "email")
 
     @property
     def SESSION_COOKIE_AGE(self):
@@ -243,7 +294,8 @@ class AppSettings(object):
         Deprecated -- use Django's settings.SESSION_COOKIE_AGE instead
         """
         from django.conf import settings
-        return self._setting('SESSION_COOKIE_AGE', settings.SESSION_COOKIE_AGE)
+
+        return self._setting("SESSION_COOKIE_AGE", settings.SESSION_COOKIE_AGE)
 
     @property
     def SESSION_REMEMBER(self):
@@ -252,18 +304,18 @@ class AppSettings(object):
         ("Remember me?"), `False` to not remember, and `True` to always
         remember.
         """
-        return self._setting('SESSION_REMEMBER', None)
+        return self._setting("SESSION_REMEMBER", None)
 
     @property
     def TEMPLATE_EXTENSION(self):
         """
         A string defining the template extension to use, defaults to `html`.
         """
-        return self._setting('TEMPLATE_EXTENSION', 'html')
+        return self._setting("TEMPLATE_EXTENSION", "html")
 
     @property
     def FORMS(self):
-        return self._setting('FORMS', {})
+        return self._setting("FORMS", {})
 
     @property
     def LOGIN_ATTEMPTS_LIMIT(self):
@@ -272,7 +324,7 @@ class AppSettings(object):
         exceeded, the user is prohibited from logging in for the
         specified `LOGIN_ATTEMPTS_TIMEOUT`
         """
-        return self._setting('LOGIN_ATTEMPTS_LIMIT', 5)
+        return self._setting("LOGIN_ATTEMPTS_LIMIT", 5)
 
     @property
     def LOGIN_ATTEMPTS_TIMEOUT(self):
@@ -281,46 +333,60 @@ class AppSettings(object):
         which the user is prohibited from trying to log in.  Defaults to
         5 minutes.
         """
-        return self._setting('LOGIN_ATTEMPTS_TIMEOUT', 60 * 5)
+        return self._setting("LOGIN_ATTEMPTS_TIMEOUT", 60 * 5)
 
     @property
     def EMAIL_CONFIRMATION_HMAC(self):
-        return self._setting('EMAIL_CONFIRMATION_HMAC', True)
+        return self._setting("EMAIL_CONFIRMATION_HMAC", True)
 
     @property
     def SALT(self):
-        return self._setting('SALT', 'account')
+        return self._setting("SALT", "account")
 
     @property
     def PRESERVE_USERNAME_CASING(self):
-        return self._setting('PRESERVE_USERNAME_CASING', True)
+        return self._setting("PRESERVE_USERNAME_CASING", True)
 
     @property
     def USERNAME_VALIDATORS(self):
         from django.core.exceptions import ImproperlyConfigured
-        from allauth.utils import import_attribute
-        from allauth.utils import get_user_model
 
-        path = self._setting('USERNAME_VALIDATORS', None)
+        from allauth.utils import get_user_model, import_attribute
+
+        path = self._setting("USERNAME_VALIDATORS", None)
         if path:
             ret = import_attribute(path)
             if not isinstance(ret, list):
                 raise ImproperlyConfigured(
-                    'ACCOUNT_USERNAME_VALIDATORS is expected to be a list')
+                    "ACCOUNT_USERNAME_VALIDATORS is expected to be a list"
+                )
         else:
             if self.USER_MODEL_USERNAME_FIELD is not None:
-                ret = get_user_model()._meta.get_field(
-                    self.USER_MODEL_USERNAME_FIELD).validators
+                ret = (
+                    get_user_model()
+                    ._meta.get_field(self.USER_MODEL_USERNAME_FIELD)
+                    .validators
+                )
             else:
                 ret = []
         return ret
 
+    @property
+    def PASSWORD_RESET_TOKEN_GENERATOR(self):
+        from allauth.account.forms import EmailAwarePasswordResetTokenGenerator
+        from allauth.utils import import_attribute
 
-# Ugly? Guido recommends this himself ...
-# http://mail.python.org/pipermail/python-ideas/2012-May/014969.html
-import sys  # noqa
+        token_generator_path = self._setting("PASSWORD_RESET_TOKEN_GENERATOR", None)
+        if token_generator_path is not None:
+            token_generator = import_attribute(token_generator_path)
+        else:
+            token_generator = EmailAwarePasswordResetTokenGenerator
+        return token_generator
 
 
-app_settings = AppSettings('ACCOUNT_')
-app_settings.__name__ = __name__
-sys.modules[__name__] = app_settings
+_app_settings = AppSettings("ACCOUNT_")
+
+
+def __getattr__(name):
+    # See https://peps.python.org/pep-0562/
+    return getattr(_app_settings, name)
