@@ -1,6 +1,5 @@
 from __future__ import absolute_import
 
-import warnings
 from importlib import import_module
 
 from django import forms
@@ -47,7 +46,7 @@ class EmailAwarePasswordResetTokenGenerator(PasswordResetTokenGenerator):
         return ret
 
 
-default_token_generator = EmailAwarePasswordResetTokenGenerator()
+default_token_generator = app_settings.PASSWORD_RESET_TOKEN_GENERATOR()
 
 
 class PasswordVerificationMixin(object):
@@ -88,7 +87,6 @@ class SetPasswordField(PasswordField):
 
 
 class LoginForm(forms.Form):
-
     password = PasswordField(label=_("Password"), autocomplete="current-password")
     remember = forms.BooleanField(label=_("Remember Me"), required=False)
 
@@ -96,7 +94,7 @@ class LoginForm(forms.Form):
     error_messages = {
         "account_inactive": _("This account is currently inactive."),
         "email_password_mismatch": _(
-            "The e-mail address and/or password you specified are not correct."
+            "The email address and/or password you specified are not correct."
         ),
         "username_password_mismatch": _(
             "The username and/or password you specified are not correct."
@@ -110,11 +108,11 @@ class LoginForm(forms.Form):
             login_widget = forms.TextInput(
                 attrs={
                     "type": "email",
-                    "placeholder": _("E-mail address"),
+                    "placeholder": _("Email address"),
                     "autocomplete": "email",
                 }
             )
-            login_field = forms.EmailField(label=_("E-mail"), widget=login_widget)
+            login_field = forms.EmailField(label=_("Email"), widget=login_widget)
         elif app_settings.AUTHENTICATION_METHOD == AuthenticationMethod.USERNAME:
             login_widget = forms.TextInput(
                 attrs={"placeholder": _("Username"), "autocomplete": "username"}
@@ -130,7 +128,7 @@ class LoginForm(forms.Form):
                 == AuthenticationMethod.USERNAME_EMAIL
             )
             login_widget = forms.TextInput(
-                attrs={"placeholder": _("Username or e-mail"), "autocomplete": "email"}
+                attrs={"placeholder": _("Username or email"), "autocomplete": "email"}
             )
             login_field = forms.CharField(
                 label=pgettext("field label", "Login"), widget=login_widget
@@ -251,16 +249,10 @@ def _base_signup_form_class():
             'Module "%s" does not define a' ' "%s" class' % (fc_module, fc_classname)
         )
     if not hasattr(fc_class, "signup"):
-        if hasattr(fc_class, "save"):
-            warnings.warn(
-                "The custom signup form must offer"
-                " a `def signup(self, request, user)` method",
-                DeprecationWarning,
-            )
-        else:
-            raise exceptions.ImproperlyConfigured(
-                'The custom signup form must implement a "signup" method'
-            )
+        raise exceptions.ImproperlyConfigured(
+            "The custom signup form must offer"
+            " a `def signup(self, request, user)` method",
+        )
     return fc_class
 
 
@@ -276,7 +268,7 @@ class BaseSignupForm(_base_signup_form_class()):
         widget=forms.TextInput(
             attrs={
                 "type": "email",
-                "placeholder": _("E-mail address"),
+                "placeholder": _("Email address"),
                 "autocomplete": "email",
             }
         )
@@ -287,6 +279,14 @@ class BaseSignupForm(_base_signup_form_class()):
         self.username_required = kwargs.pop(
             "username_required", app_settings.USERNAME_REQUIRED
         )
+        # We can only truly prevent enumeration if email verification is
+        # mandatory, because in that case, regardless of whether or not the
+        # email is in use, the user will always receive an email.
+        self.prevent_enumeration = app_settings.PREVENT_ENUMERATION and (
+            app_settings.EMAIL_VERIFICATION
+            == app_settings.EmailVerificationMethod.MANDATORY
+        )
+        self.account_already_exists = False
         super(BaseSignupForm, self).__init__(*args, **kwargs)
         username_field = self.fields["username"]
         username_field.max_length = get_username_max_length()
@@ -304,19 +304,19 @@ class BaseSignupForm(_base_signup_form_class()):
         ]
         if app_settings.SIGNUP_EMAIL_ENTER_TWICE:
             self.fields["email2"] = forms.EmailField(
-                label=_("E-mail (again)"),
+                label=_("Email (again)"),
                 widget=forms.TextInput(
                     attrs={
                         "type": "email",
-                        "placeholder": _("E-mail address confirmation"),
+                        "placeholder": _("Email address confirmation"),
                     }
                 ),
             )
         if email_required:
-            self.fields["email"].label = gettext("E-mail")
+            self.fields["email"].label = gettext("Email")
             self.fields["email"].required = True
         else:
-            self.fields["email"].label = gettext("E-mail (optional)")
+            self.fields["email"].label = gettext("Email (optional)")
             self.fields["email"].required = False
             self.fields["email"].widget.is_required = False
             if self.username_required:
@@ -338,13 +338,22 @@ class BaseSignupForm(_base_signup_form_class()):
     def clean_username(self):
         value = self.cleaned_data["username"]
         value = get_adapter().clean_username(value)
+        # Note regarding preventing enumeration: if the username is already
+        # taken, but the email address is not, we would still leak information
+        # if we were to send an email to that email address stating that the
+        # username is already in use.
         return value
 
     def clean_email(self):
         value = self.cleaned_data["email"]
         value = get_adapter().clean_email(value)
         if value and app_settings.UNIQUE_EMAIL:
-            value = self.validate_unique_email(value)
+            try:
+                value = self.validate_unique_email(value)
+            except forms.ValidationError:
+                if not self.prevent_enumeration:
+                    raise
+                self.account_already_exists = True
         return value
 
     def validate_unique_email(self, value):
@@ -360,18 +369,25 @@ class BaseSignupForm(_base_signup_form_class()):
         return cleaned_data
 
     def custom_signup(self, request, user):
-        custom_form = super(BaseSignupForm, self)
-        if hasattr(custom_form, "signup") and callable(custom_form.signup):
-            custom_form.signup(request, user)
+        self.signup(request, user)
+
+    def try_save(self, request):
+        """Try and save te user. This can fail in case of a conflict on the
+        email address, in that case we will send an "account already exists"
+        email and return a standard "email verification sent" response.
+        """
+        if self.account_already_exists:
+            # Don't create a new acount, only send an email informing the user
+            # that (s)he already has one...
+            email = self.cleaned_data["email"]
+            adapter = get_adapter(request)
+            adapter.send_account_already_exists_mail(email)
+            user = None
+            resp = adapter.respond_email_verification_sent(request, None)
         else:
-            warnings.warn(
-                "The custom signup form must offer"
-                " a `def signup(self, request, user)` method",
-                DeprecationWarning,
-            )
-            # Historically, it was called .save, but this is confusing
-            # in case of ModelForm
-            custom_form.save(user)
+            user = self.save(request)
+            resp = None
+        return user, resp
 
 
 class SignupForm(BaseSignupForm):
@@ -393,7 +409,7 @@ class SignupForm(BaseSignupForm):
 
         # `password` cannot be of type `SetPasswordField`, as we don't
         # have a `User` yet. So, let's populate a dummy user to be used
-        # for password validaton.
+        # for password validation.
         User = get_user_model()
         dummy_user = User()
         user_username(dummy_user, self.cleaned_data.get("username"))
@@ -418,6 +434,8 @@ class SignupForm(BaseSignupForm):
         return self.cleaned_data
 
     def save(self, request):
+        if self.account_already_exists:
+            raise ValueError(self.cleaned_data.get("email"))
         adapter = get_adapter(request)
         user = adapter.new_user(request)
         adapter.save_user(request, user, self)
@@ -434,12 +452,11 @@ class UserForm(forms.Form):
 
 
 class AddEmailForm(UserForm):
-
     email = forms.EmailField(
-        label=_("E-mail"),
+        label=_("Email"),
         required=True,
         widget=forms.TextInput(
-            attrs={"type": "email", "placeholder": _("E-mail address")}
+            attrs={"type": "email", "placeholder": _("Email address")}
         ),
     )
 
@@ -448,21 +465,15 @@ class AddEmailForm(UserForm):
         value = get_adapter().clean_email(value)
         errors = {
             "this_account": _(
-                "This e-mail address is already associated with this account."
+                "This email address is already associated with this account."
             ),
-            "different_account": _(
-                "This e-mail address is already associated with another account."
-            ),
-            "max_email_addresses": _("You cannot add more than %d e-mail addresses."),
+            "max_email_addresses": _("You cannot add more than %d email addresses."),
         }
         users = filter_users_by_email(value)
         on_this_account = [u for u in users if u.pk == self.user.pk]
-        on_diff_account = [u for u in users if u.pk != self.user.pk]
 
         if on_this_account:
             raise forms.ValidationError(errors["this_account"])
-        if on_diff_account and app_settings.UNIQUE_EMAIL:
-            raise forms.ValidationError(errors["different_account"])
         if not EmailAddress.objects.can_add_email(self.user):
             raise forms.ValidationError(
                 errors["max_email_addresses"] % app_settings.MAX_EMAIL_ADDRESSES
@@ -470,13 +481,16 @@ class AddEmailForm(UserForm):
         return value
 
     def save(self, request):
+        if app_settings.CHANGE_EMAIL:
+            return EmailAddress.objects.add_new_email(
+                request, self.user, self.cleaned_data["email"]
+            )
         return EmailAddress.objects.add_email(
             request, self.user, self.cleaned_data["email"], confirm=True
         )
 
 
 class ChangePasswordForm(PasswordVerificationMixin, UserForm):
-
     oldpassword = PasswordField(
         label=_("Current Password"), autocomplete="current-password"
     )
@@ -497,7 +511,6 @@ class ChangePasswordForm(PasswordVerificationMixin, UserForm):
 
 
 class SetPasswordForm(PasswordVerificationMixin, UserForm):
-
     password1 = SetPasswordField(label=_("Password"))
     password2 = PasswordField(label=_("Password (again)"))
 
@@ -510,14 +523,13 @@ class SetPasswordForm(PasswordVerificationMixin, UserForm):
 
 
 class ResetPasswordForm(forms.Form):
-
     email = forms.EmailField(
-        label=_("E-mail"),
+        label=_("Email"),
         required=True,
         widget=forms.TextInput(
             attrs={
                 "type": "email",
-                "placeholder": _("E-mail address"),
+                "placeholder": _("Email address"),
                 "autocomplete": "email",
             }
         ),
@@ -526,10 +538,10 @@ class ResetPasswordForm(forms.Form):
     def clean_email(self):
         email = self.cleaned_data["email"]
         email = get_adapter().clean_email(email)
-        self.users = filter_users_by_email(email, is_active=True)
+        self.users = filter_users_by_email(email, is_active=True, prefer_verified=True)
         if not self.users and not app_settings.PREVENT_ENUMERATION:
             raise forms.ValidationError(
-                _("The e-mail address is not assigned to any user account")
+                _("The email address is not assigned to any user account")
             )
         return self.cleaned_data["email"]
 
@@ -555,7 +567,6 @@ class ResetPasswordForm(forms.Form):
         token_generator = kwargs.get("token_generator", default_token_generator)
 
         for user in users:
-
             temp_key = token_generator.make_token(user)
 
             # save it to the password reset model
@@ -563,9 +574,10 @@ class ResetPasswordForm(forms.Form):
             # password_reset.save()
 
             # send the password reset email
+            uid = user_pk_to_url_str(user)
             path = reverse(
                 "account_reset_password_from_key",
-                kwargs=dict(uidb36=user_pk_to_url_str(user), key=temp_key),
+                kwargs=dict(uidb36=uid, key=temp_key),
             )
             url = build_absolute_uri(request, path)
 
@@ -573,6 +585,8 @@ class ResetPasswordForm(forms.Form):
                 "current_site": get_current_site(request),
                 "user": user,
                 "password_reset_url": url,
+                "uid": uid,
+                "key": temp_key,
                 "request": request,
             }
 
@@ -584,7 +598,6 @@ class ResetPasswordForm(forms.Form):
 
 
 class ResetPasswordKeyForm(PasswordVerificationMixin, forms.Form):
-
     password1 = SetPasswordField(label=_("New Password"))
     password2 = PasswordField(label=_("New Password (again)"))
 
@@ -599,7 +612,6 @@ class ResetPasswordKeyForm(PasswordVerificationMixin, forms.Form):
 
 
 class UserTokenForm(forms.Form):
-
     uidb36 = forms.CharField()
     key = forms.CharField()
 
