@@ -1,13 +1,12 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-from django.db import migrations, models
+from django.db import migrations
 
 import allauth.socialaccount.fields
 
 
 class Migration(migrations.Migration):
-
     dependencies = [
         ("socialaccount", "0002_token_max_lengths"),
     ]
