@@ -1,3 +1,5 @@
+from django.core.exceptions import ImproperlyConfigured
+
 from allauth.account.models import EmailAddress
 from allauth.socialaccount import app_settings
 from allauth.socialaccount.adapter import get_adapter
@@ -8,11 +10,14 @@ class ProviderException(Exception):
 
 
 class Provider(object):
-
     slug = None
+    uses_apps = True
 
-    def __init__(self, request):
+    def __init__(self, request, app=None):
         self.request = request
+        if self.uses_apps and app is None:
+            raise ValueError("missing: app")
+        self.app = app
 
     @classmethod
     def get_slug(cls):
@@ -25,10 +30,6 @@ class Provider(object):
         """
         raise NotImplementedError("get_login_url() for " + self.name)
 
-    def get_app(self, request, config=None):
-        adapter = get_adapter(request)
-        return adapter.get_app(request, self.id, config=config)
-
     def media_js(self, request):
         """
         Some providers may require extra scripts (e.g. a Facebook connect)
@@ -62,11 +63,26 @@ class Provider(object):
 
         adapter = get_adapter(request)
         uid = self.extract_uid(response)
+        if not isinstance(uid, str):
+            raise ValueError(f"uid must be a string: {repr(uid)}")
+        if len(uid) > app_settings.UID_MAX_LENGTH:
+            raise ImproperlyConfigured(
+                f"SOCIALACCOUNT_UID_MAX_LENGTH too small (<{len(uid)})"
+            )
+
         extra_data = self.extract_extra_data(response)
         common_fields = self.extract_common_fields(response)
-        socialaccount = SocialAccount(extra_data=extra_data, uid=uid, provider=self.id)
+        socialaccount = SocialAccount(
+            extra_data=extra_data,
+            uid=uid,
+            provider=self.app.provider_id or self.app.provider,
+        )
         email_addresses = self.extract_email_addresses(response)
-        self.cleanup_email_addresses(common_fields.get("email"), email_addresses)
+        self.cleanup_email_addresses(
+            common_fields.get("email"),
+            email_addresses,
+            email_verified=common_fields.get("email_verified"),
+        )
         sociallogin = SocialLogin(
             account=socialaccount, email_addresses=email_addresses
         )
@@ -106,10 +122,12 @@ class Provider(object):
         """
         return {}
 
-    def cleanup_email_addresses(self, email, addresses):
+    def cleanup_email_addresses(self, email, addresses, email_verified=False):
         # Move user.email over to EmailAddress
         if email and email.lower() not in [a.email.lower() for a in addresses]:
-            addresses.append(EmailAddress(email=email, verified=False, primary=True))
+            addresses.append(
+                EmailAddress(email=email, verified=bool(email_verified), primary=True)
+            )
         # Force verified emails
         settings = self.get_settings()
         verified_email = settings.get("VERIFIED_EMAIL", False)
