@@ -1,10 +1,13 @@
 import os
 
+from django.contrib.auth.hashers import PBKDF2PasswordHasher
+
 
 SECRET_KEY = "psst"
 SITE_ID = 1
 ALLOWED_HOSTS = ("*",)
 USE_I18N = False
+USE_TZ = True
 
 DATABASES = {
     "default": {
@@ -22,7 +25,9 @@ ROOT_URLCONF = "allauth.urls"
 TEMPLATES = [
     {
         "BACKEND": "django.template.backends.django.DjangoTemplates",
-        "DIRS": [os.path.join(os.path.dirname(__file__), "example", "example", "templates")],
+        "DIRS": [
+            os.path.join(os.path.dirname(__file__), "example", "example", "templates")
+        ],
         "APP_DIRS": True,
         "OPTIONS": {
             "context_processors": [
@@ -77,6 +82,7 @@ INSTALLED_APPS = (
     "allauth.socialaccount.providers.dataporten",
     "allauth.socialaccount.providers.daum",
     "allauth.socialaccount.providers.digitalocean",
+    "allauth.socialaccount.providers.dingtalk",
     "allauth.socialaccount.providers.discord",
     "allauth.socialaccount.providers.disqus",
     "allauth.socialaccount.providers.douban",
@@ -106,7 +112,7 @@ INSTALLED_APPS = (
     "allauth.socialaccount.providers.google",
     "allauth.socialaccount.providers.gumroad",
     "allauth.socialaccount.providers.hubic",
-    'allauth.socialaccount.providers.hubspot',
+    "allauth.socialaccount.providers.hubspot",
     "allauth.socialaccount.providers.instagram",
     "allauth.socialaccount.providers.jupyterhub",
     "allauth.socialaccount.providers.kakao",
@@ -120,22 +126,26 @@ INSTALLED_APPS = (
     "allauth.socialaccount.providers.mediawiki",
     "allauth.socialaccount.providers.meetup",
     "allauth.socialaccount.providers.microsoft",
+    "allauth.socialaccount.providers.miro",
     "allauth.socialaccount.providers.naver",
     "allauth.socialaccount.providers.netiq",
     "allauth.socialaccount.providers.nextcloud",
+    "allauth.socialaccount.providers.notion",
     "allauth.socialaccount.providers.odnoklassniki",
     "allauth.socialaccount.providers.openid",
+    "allauth.socialaccount.providers.openid_connect",
     "allauth.socialaccount.providers.openstreetmap",
     "allauth.socialaccount.providers.orcid",
     "allauth.socialaccount.providers.patreon",
     "allauth.socialaccount.providers.paypal",
-    "allauth.socialaccount.providers.persona",
     "allauth.socialaccount.providers.pinterest",
     "allauth.socialaccount.providers.pocket",
+    "allauth.socialaccount.providers.questrade",
     "allauth.socialaccount.providers.quickbooks",
     "allauth.socialaccount.providers.reddit",
     "allauth.socialaccount.providers.robinhood",
     "allauth.socialaccount.providers.salesforce",
+    "allauth.socialaccount.providers.saml",
     "allauth.socialaccount.providers.sharefile",
     "allauth.socialaccount.providers.shopify",
     "allauth.socialaccount.providers.slack",
@@ -154,10 +164,12 @@ INSTALLED_APPS = (
     "allauth.socialaccount.providers.twentythreeandme",
     "allauth.socialaccount.providers.twitch",
     "allauth.socialaccount.providers.twitter",
+    "allauth.socialaccount.providers.twitter_oauth2",
     "allauth.socialaccount.providers.untappd",
     "allauth.socialaccount.providers.vimeo",
     "allauth.socialaccount.providers.vimeo_oauth2",
     "allauth.socialaccount.providers.vk",
+    "allauth.socialaccount.providers.wahoo",
     "allauth.socialaccount.providers.weibo",
     "allauth.socialaccount.providers.weixin",
     "allauth.socialaccount.providers.windowslive",
@@ -179,8 +191,6 @@ AUTHENTICATION_BACKENDS = (
 STATIC_ROOT = "/tmp/"  # Dummy
 STATIC_URL = "/static/"
 
-from django.contrib.auth.hashers import PBKDF2PasswordHasher
-
 
 class MyPBKDF2PasswordHasher(PBKDF2PasswordHasher):
     """
@@ -203,3 +213,29 @@ PASSWORD_HASHERS = [
 
 ACCOUNT_EMAIL_CONFIRMATION_COOLDOWN = 0
 ACCOUNT_RATE_LIMITS = {}
+
+
+SOCIALACCOUNT_PROVIDERS = {
+    "openid_connect": {
+        "APPS": [
+            {
+                "provider_id": "unittest-server",
+                "name": "Unittest Server",
+                "client_id": "Unittest client_id",
+                "client_secret": "Unittest client_secret",
+                "settings": {
+                    "server_url": "https://unittest.example.com",
+                },
+            },
+            {
+                "provider_id": "other-server",
+                "name": "Other Example Server",
+                "client_id": "other client_id",
+                "client_secret": "other client_secret",
+                "settings": {
+                    "server_url": "https://other.example.com",
+                },
+            },
+        ],
+    }
+}
